"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.assertProgramNode = exports.isProgramNode = exports.ProgramNode = void 0;
const utils_1 = require("../utils");
const AccountNode_1 = require("./AccountNode");
const DefinedTypeNode_1 = require("./DefinedTypeNode");
const ErrorNode_1 = require("./ErrorNode");
const InstructionNode_1 = require("./InstructionNode");
class ProgramNode {
    constructor(metadata, accounts, instructions, definedTypes, errors) {
        this.nodeClass = 'ProgramNode';
        this.metadata = {
            ...metadata,
            name: (0, utils_1.mainCase)(metadata.name),
            prefix: (0, utils_1.mainCase)(metadata.prefix),
        };
        this.accounts = accounts;
        this.instructions = instructions;
        this.definedTypes = definedTypes;
        this.errors = errors;
    }
    static fromIdl(idl) {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
        const origin = (_b = (_a = idl.metadata) === null || _a === void 0 ? void 0 : _a.origin) !== null && _b !== void 0 ? _b : null;
        const accounts = ((_c = idl.accounts) !== null && _c !== void 0 ? _c : []).map(AccountNode_1.AccountNode.fromIdl);
        const instructions = ((_d = idl.instructions) !== null && _d !== void 0 ? _d : []).map((ix) => {
            var _a;
            if (origin === 'anchor') {
                return InstructionNode_1.InstructionNode.fromIdl({
                    ...ix,
                    defaultOptionalAccounts: (_a = ix.defaultOptionalAccounts) !== null && _a !== void 0 ? _a : true,
                });
            }
            return InstructionNode_1.InstructionNode.fromIdl(ix);
        });
        const definedTypes = ((_e = idl.types) !== null && _e !== void 0 ? _e : []).map(DefinedTypeNode_1.DefinedTypeNode.fromIdl);
        const errors = ((_f = idl.errors) !== null && _f !== void 0 ? _f : []).map(ErrorNode_1.ErrorNode.fromIdl);
        const metadata = {
            name: (_g = idl.name) !== null && _g !== void 0 ? _g : '',
            prefix: '',
            publicKey: (_j = (_h = idl.metadata) === null || _h === void 0 ? void 0 : _h.address) !== null && _j !== void 0 ? _j : '',
            version: (_k = idl.version) !== null && _k !== void 0 ? _k : '',
            origin,
            idl,
            internal: false,
        };
        return new ProgramNode(metadata, accounts, instructions, definedTypes, errors);
    }
    accept(visitor) {
        return visitor.visitProgram(this);
    }
    get name() {
        return this.metadata.name;
    }
    get instructionsWithSubs() {
        return this.instructions.flatMap((instruction) => [
            instruction,
            ...instruction.getAllSubInstructions(),
        ]);
    }
}
exports.ProgramNode = ProgramNode;
function isProgramNode(node) {
    return !!node && node.nodeClass === 'ProgramNode';
}
exports.isProgramNode = isProgramNode;
function assertProgramNode(node) {
    var _a;
    if (!isProgramNode(node)) {
        throw new Error(`Expected ProgramNode, got ${(_a = node === null || node === void 0 ? void 0 : node.nodeClass) !== null && _a !== void 0 ? _a : 'null'}.`);
    }
}
exports.assertProgramNode = assertProgramNode;
//# sourceMappingURL=ProgramNode.js.map